@@ -11,71 +11,6 @@ from iman import *
 _system = platform.system().lower()
 
 
-ALawDecompressTable =[
-
-     -5504, -5248, -6016, -5760, -4480, -4224, -4992, -4736,
-
-     -7552, -7296, -8064, -7808, -6528, -6272, -7040, -6784,
-
-     -2752, -2624, -3008, -2880, -2240, -2112, -2496, -2368,
-
-     -3776, -3648, -4032, -3904, -3264, -3136, -3520, -3392,
-
-     -22016,-20992,-24064,-23040,-17920,-16896,-19968,-18944,
-
-     -30208,-29184,-32256,-31232,-26112,-25088,-28160,-27136,
-
-     -11008,-10496,-12032,-11520,-8960, -8448, -9984, -9472,
-
-     -15104,-14592,-16128,-15616,-13056,-12544,-14080,-13568,
-
-     -344,  -328,  -376,  -360,  -280,  -264,  -312,  -296,
-
-     -472,  -456,  -504,  -488,  -408,  -392,  -440,  -424,
-
-     -88,   -72,   -120,  -104,  -24,   -8,    -56,   -40,
-
-     -216,  -200,  -248,  -232,  -152,  -136,  -184,  -168,
-
-     -1376, -1312, -1504, -1440, -1120, -1056, -1248, -1184,
-
-     -1888, -1824, -2016, -1952, -1632, -1568, -1760, -1696,
-
-     -688,  -656,  -752,  -720,  -560,  -528,  -624,  -592,
-
-     -944,  -912,  -1008, -976,  -816,  -784,  -880,  -848,
-
-      5504,  5248,  6016,  5760,  4480,  4224,  4992,  4736,
-
-      7552,  7296,  8064,  7808,  6528,  6272,  7040,  6784,
-
-      2752,  2624,  3008,  2880,  2240,  2112,  2496,  2368,
-
-      3776,  3648,  4032,  3904,  3264,  3136,  3520,  3392,
-
-      22016, 20992, 24064, 23040, 17920, 16896, 19968, 18944,
-
-      30208, 29184, 32256, 31232, 26112, 25088, 28160, 27136,
-
-      11008, 10496, 12032, 11520, 8960,  8448,  9984,  9472,
-
-      15104, 14592, 16128, 15616, 13056, 12544, 14080, 13568,
-
-      344,   328,   376,   360,   280,   264,   312,   296,
-
-      472,   456,   504,   488,   408,   392,   440,   424,
-
-      88,    72,   120,   104,    24,     8,    56,    40,
-
-      216,   200,   248,   232,   152,   136,   184,   168,
-
-      1376,  1312,  1504,  1440,  1120,  1056,  1248,  1184,
-
-      1888,  1824,  2016,  1952,  1632,  1568,  1760,  1696,
-
-      688,   656,   752,   720,   560,   528,   624,   592,
-
-      944,   912,  1008,   976,   816,   784,   880,   848]
 
 def Read_Alaw_1(filename):
     with open(filename, "rb") as binaryfile :
@@ -84,8 +19,11 @@ def Read_Alaw_1(filename):
     return np.array(g , dtype=np.float32)
     
     
-def Read_Alaw(filename,sr=8000,ffmpeg_path='c:\\ffmpeg.exe'):
-           ffmpeg_command =  '%s -ar 8000 -f alaw -i "%s" -ac 1 -ar %s -f wav pipe:1' %(ffmpeg_path ,filename,sr)
+def Read_Alaw(filename,sr=8000,start_from=0,dur=-1,ffmpeg_path='c:\\ffmpeg.exe'):
+
+           ffmpeg_command = [ffmpeg_path, "-ss" , str(start_from) ,  "-ar","8000", "-f" ,"alaw" ,'-i', filename, '-ac', "1", '-ar', str(sr), '-f', 'wav', 'pipe:1']
+           if (dur!=-1):
+               ffmpeg_command = [ffmpeg_path, "-ss" , str(start_from) , "-t" ,str(dur) , "-ar","8000", "-f" ,"alaw" ,'-i', filename, '-ac', "1", '-ar', str(sr), '-f', 'wav', 'pipe:1']            
            
            pipe = subprocess.run(ffmpeg_command, stdout=subprocess.PIPE,   stderr=subprocess.PIPE,  bufsize=10**8)
            
@@ -149,18 +87,36 @@ def ReadMp3_miniaudio(filename,sr,mono=True):
     return(data)
 
 
-def ReadMp3(filename,sr=16000,mono=True,ffmpeg_path='c:\\ffmpeg.exe'):
-     chnum=2
-     if (mono):
-        chnum=1
-     
-     if (_system == 'windows'):      
-           ffmpeg_command =  '%s -i "%s" -ac %s -ar %s -f wav pipe:1' %(ffmpeg_path ,filename, chnum,sr)
+
+
+def ReadFFMPEG(filename,sr=16000,start_from=0,dur=-1,mono = False , ffmpeg_path='c:\\ffmpeg.exe',ffprobe_path='c:\\ffprobe.exe'):
+           
+           chnumber = 1
+           if (not mono):
+             ffprobe_command = [ffprobe_path ,  "-v","error","-show_entries","stream=channels","-of","default=nw=1" ,filename ] 
+             pipe1 = subprocess.run(ffprobe_command, stdout=subprocess.PIPE)
+             chnumber = int(str(pipe1.stdout).strip().split('=')[1].split('\\')[0])
+             
+           
+           
+           ffmpeg_command = [ffmpeg_path,  '-i', filename,'-ss' , str(start_from),  '-ac' ,str(chnumber) , '-ar', str(sr), '-f', 'wav', 'pipe:1']
+           
+           if (dur!=-1):
+                    ffmpeg_command = [ffmpeg_path, '-i', filename, '-ss' , str(start_from), '-t' , str(dur) , '-ac', str(chnumber), '-ar', str(sr), '-f', 'wav', 'pipe:1']
            
            pipe = subprocess.run(ffmpeg_command, stdout=subprocess.PIPE,   stderr=subprocess.PIPE,  bufsize=10**8)
            
-           audio_np = np.frombuffer(buffer=pipe.stdout, dtype=np.uint16, offset=8*44)
+           # audio_np = np.frombuffer(buffer=pipe.stdout, dtype=np.uint16, offset=8*44)
              
+             
+           import wave
+           import io
+           wav_data = io.BytesIO(pipe.stdout)
+           with wave.open(wav_data) as wav_file:
+               audio_data = wav_file.readframes(wav_file.getnframes())
+               audio_np = np.frombuffer(audio_data, dtype=np.int16)
+               
+               
            x =np.where(audio_np>32768)    [0]    
            y =np.where(audio_np<=32768)    [0]  
            
@@ -168,21 +124,22 @@ def ReadMp3(filename,sr=16000,mono=True,ffmpeg_path='c:\\ffmpeg.exe'):
            out_wav[x] = (audio_np[x] - 32768)/32768-1
            out_wav[y] = audio_np[y] /32768
            
-           if (mono):
-              return(out_wav) 
+           
+           x=[]
+           if (chnumber==1):
+              x.append(out_wav)
+              if (mono):
+                 return x[0]
+              else:   
+                 return(x) 
            else:
-              ch1 = out_wav[::2]
-              ch2=out_wav[1::2]
-              return(ch1,ch2)
-     else:
-        chnum=1
-        ffmpeg_command =  'ffmpeg -i "%s" -ac %s -ar %s -f wav "%s"' %(filename, chnum,sr , PN(filename) + '_temp.wav')
-        os.system(ffmpeg_command) 
-        x = Read(PN(filename) + '_temp.wav' , sr)
-        os.system('rm "%s"' %(PN(filename) + '_temp.wav'))
-        return( x )       
+              x.append(out_wav[1::2])
+              x.append(out_wav[::2]) 
+              return(x)
 
-def Read(filename,sr=8000,ffmpeg_path='c:\\ffmpeg.exe'):   # output sampling rate is sr
+
+
+def Read(filename,sr=8000, start_from=0, dur=-1,mono = False,ffmpeg_path='c:\\ffmpeg.exe',ffprobe_path='c:\\ffprobe.exe' ):   # output sampling rate is sr
    """Return Audio Data. (Just Mono Files)
 
     Parameters
@@ -196,11 +153,11 @@ def Read(filename,sr=8000,ffmpeg_path='c:\\ffmpeg.exe'):   # output sampling rat
 
    """   
    ext = os.path.basename(filename).split('.')[-1] 
-   if (ext.lower() == 'l' or ext.lower() == 'r' ):
-         return( Read_Alaw(filename,sr=sr ,ffmpeg_path=ffmpeg_path ))
+   if (ext.lower() == 'l' or ext.lower() == 'r' or os.path.basename(filename).lower().endswith('.r.wav') or os.path.basename(filename).lower().endswith('.l.wav') or os.path.basename(filename).lower().endswith('.rtemp.wav') or os.path.basename(filename).lower().endswith('.ltemp.wav')):
+         return( Read_Alaw(filename,sr=sr ,ffmpeg_path=ffmpeg_path ,start_from=start_from,dur=dur))
        
   
-   return(ReadMp3(filename,sr=sr, mono=True,ffmpeg_path=ffmpeg_path))
+   return(ReadFFMPEG(filename,sr=sr, start_from=start_from,dur=dur,mono =mono ,  ffmpeg_path=ffmpeg_path,ffprobe_path=ffprobe_path ))
     
    
    
@@ -260,6 +217,25 @@ def Write(filename, data ,fs):
 
    """
    wave.write(filename, fs, np.int16(data*32768))
+   
+def WriteS(filename, data ,fs):
+
+   """Write Audio Data Sterio.
+
+    Parameters
+    ----------
+    filename : Output file path (.wav)
+    data : Audio data
+    fs : sampling rate
+    
+    Output
+    ----------
+    Nothing
+
+   """
+   xx = np.asarray( [np.int16(data*32768) , np.int16(data*32768)]).transpose()
+
+   wave.write(filename, fs, xx)
 def rmse(y=None, S=None, frame_length=2048, hop_length=512,
          center=True, pad_mode='reflect'):
     
@@ -420,9 +396,9 @@ def change_format(fname , sr=16000 , ext='mp3' , mono=True ,ffmpeg_path='c:\\ffm
     if (mono):
       chnum=1
       
-    forext = ''
+    forext = []
     if (PE(fname).lower()=='.l' or PE(fname).lower()=='.r') :
-         forext = "-ar 8000 -f alaw"     
+         forext = ['-ar', '8000', '-f' ,'alaw']    
       
       
     if (not ext.startswith('.')):
@@ -441,7 +417,14 @@ def change_format(fname , sr=16000 , ext='mp3' , mono=True ,ffmpeg_path='c:\\ffm
        return 0
        
     try:
-      ffmpeg_command =  '%s %s -i "%s" -ac %s -ar %s "%s"' %(ffmpeg_path , forext,fname, chnum,sr ,outfile) 
+    
+      if (forext==[]):
+         ffmpeg_command = [ffmpeg_path, '-i', fname, '-ac', str(chnum), '-ar', str(sr), 'outfile']
+      else:
+          ffmpeg_command = [ffmpeg_path]
+          ffmpeg_command.extend(forext)
+          ffmpeg_command.extend(['-i', fname, '-ac', str(chnum), '-ar', str(sr), 'outfile'])
+
       x = cmd(ffmpeg_command )
       if (PX(outfile) and PS(outfile)>1):
          return 1
@@ -512,4 +495,72 @@ def clip_value(wav):
     x = len(np.where(c==True)[0])
         
     clip_percentage = 100*x / len(wav)
-    return (clip_percentage)         
\ No newline at end of file
+    return (clip_percentage)         
+    
+    
+
+ALawDecompressTable =[
+
+     -5504, -5248, -6016, -5760, -4480, -4224, -4992, -4736,
+
+     -7552, -7296, -8064, -7808, -6528, -6272, -7040, -6784,
+
+     -2752, -2624, -3008, -2880, -2240, -2112, -2496, -2368,
+
+     -3776, -3648, -4032, -3904, -3264, -3136, -3520, -3392,
+
+     -22016,-20992,-24064,-23040,-17920,-16896,-19968,-18944,
+
+     -30208,-29184,-32256,-31232,-26112,-25088,-28160,-27136,
+
+     -11008,-10496,-12032,-11520,-8960, -8448, -9984, -9472,
+
+     -15104,-14592,-16128,-15616,-13056,-12544,-14080,-13568,
+
+     -344,  -328,  -376,  -360,  -280,  -264,  -312,  -296,
+
+     -472,  -456,  -504,  -488,  -408,  -392,  -440,  -424,
+
+     -88,   -72,   -120,  -104,  -24,   -8,    -56,   -40,
+
+     -216,  -200,  -248,  -232,  -152,  -136,  -184,  -168,
+
+     -1376, -1312, -1504, -1440, -1120, -1056, -1248, -1184,
+
+     -1888, -1824, -2016, -1952, -1632, -1568, -1760, -1696,
+
+     -688,  -656,  -752,  -720,  -560,  -528,  -624,  -592,
+
+     -944,  -912,  -1008, -976,  -816,  -784,  -880,  -848,
+
+      5504,  5248,  6016,  5760,  4480,  4224,  4992,  4736,
+
+      7552,  7296,  8064,  7808,  6528,  6272,  7040,  6784,
+
+      2752,  2624,  3008,  2880,  2240,  2112,  2496,  2368,
+
+      3776,  3648,  4032,  3904,  3264,  3136,  3520,  3392,
+
+      22016, 20992, 24064, 23040, 17920, 16896, 19968, 18944,
+
+      30208, 29184, 32256, 31232, 26112, 25088, 28160, 27136,
+
+      11008, 10496, 12032, 11520, 8960,  8448,  9984,  9472,
+
+      15104, 14592, 16128, 15616, 13056, 12544, 14080, 13568,
+
+      344,   328,   376,   360,   280,   264,   312,   296,
+
+      472,   456,   504,   488,   408,   392,   440,   424,
+
+      88,    72,   120,   104,    24,     8,    56,    40,
+
+      216,   200,   248,   232,   152,   136,   184,   168,
+
+      1376,  1312,  1504,  1440,  1120,  1056,  1248,  1184,
+
+      1888,  1824,  2016,  1952,  1632,  1568,  1760,  1696,
+
+      688,   656,   752,   720,   560,   528,   624,   592,
+
+      944,   912,  1008,   976,   816,   784,   880,   848]
\ No newline at end of file
