@@ -4,7 +4,7 @@ from functools import partial
 
 def par(files , func , worker=4 , args=[]):
   with Pool(processes=worker) as pool:
-     out_res = pool.map(partial(func, b=args), files)
+     out_res = pool.map(partial(func, _args=args), files)
   return  out_res 
 
   
\ No newline at end of file
