@@ -29,16 +29,11 @@ import os
 # os.environ["CUDA_DEVICE_ORDER"]= '0'
 import sys
 import math
-
+from iman import Audio
 import numpy as np
 from tensorflow import keras
-import tensorflow as tf
 from tensorflow.compat.v1.keras.backend import set_session
-config = tf.compat.v1.ConfigProto()
-config.gpu_options.allow_growth = True 
-config.log_device_placement = True 
-sess = tf.compat.v1.Session(config=config)
-set_session(sess) 
+
 from .thread_returning import ThreadReturning
 
 import shutil
@@ -65,13 +60,19 @@ def _energy_activity(loge, ratio=0.4):   ##########0.9
 
 #exp(150, cost0=-5)
 
-def filter_sig(isig , wav , sr):
+def filter_sig(isig , wav , sr=16000):
+    
+    if (sr!=16000):
+       wav = Audio.Resample(wav , 16000, sr)
+    
+    
     try:
       w=[]
       wn=[]
       wn.append(wav[0 : int(isig[0][1]*sr)])
-      for i , [_,a,b,_] in enumerate(isig): 
-          
+      for i , xxx in enumerate(isig): 
+          a=xxx[1]
+          b=xxx[2]
           w.append(wav[int(a*sr) : int(b*sr)])
           try:
             wn.append(wav[ int(isig[i][2]*sr) :  int(isig[i+1][1]*sr)])
@@ -107,7 +108,8 @@ def filter_output(isig , max_silence=1 ,ignore_small_speech_segments=0.5 , max_s
    
    # if (len(isig)==0):
      # return -1
-      
+    
+   isig = [list(x) for x in isig]    
   
    for i in range(len(isig)-1):
       t = isig[i+1][1] - isig[i][2] # silence between towo chunk
@@ -119,7 +121,9 @@ def filter_output(isig , max_silence=1 ,ignore_small_speech_segments=0.5 , max_s
           
           rang = np.arange(0.01,max_silence+0.1,0.1)
           for di in rang:
-             for i , [_,_,_,_,_t] in enumerate(isig):
+             for i , xxx in enumerate(isig):
+             
+                        _t = xxx[-1]
                         if (_t==-1):
                             break                 
                         if (_t <=di):  
@@ -130,7 +134,8 @@ def filter_output(isig , max_silence=1 ,ignore_small_speech_segments=0.5 , max_s
                            except:
                                pass                           
           _dels=[]                    
-          for i , [_,_,_,_d,_] in enumerate(isig):  
+          for i , xxxx in enumerate(isig): 
+               _d = xxxx[3]           
                if (_d<=ignore_small_speech_segments) :
                         _dels.append(i)
           _dels.reverse()
@@ -143,7 +148,8 @@ def filter_output(isig , max_silence=1 ,ignore_small_speech_segments=0.5 , max_s
      
         
    isign=[]                 
-   for i , [_,_,_,_d,_] in enumerate(isig):  
+   for i , xxxxx in enumerate(isig):  
+        _d = xxxxx[3]
         if (_d> split_speech_bigger_than ) :
                
                _gc = math.ceil(_d/split_speech_bigger_than)
@@ -334,7 +340,7 @@ class Gender(DnnSegmenter):
 class Segmenter:
 
 
-    def __init__(self, vad_type = 'sad' , vad_engine='smn', detect_gender=False, sr=16000, batch_size=32 , complete_output=False,model_path="c:\\keras_speech_music_noise_cnn.hdf5"):
+    def __init__(self, vad_type = 'sad' , vad_engine='smn', detect_gender=False, sr=16000, batch_size=32 , complete_output=False,model_path="c:\\keras_speech_music_noise_cnn.hdf5",gender_path="c:\\keras_male_female_cnn.hdf5" , ffmpeg_path='c:\\ffmpeg.exe',device='cuda' ,input_type="file"):
         """
         Load neural network models
         
@@ -347,9 +353,29 @@ class Segmenter:
         
         'detect_gender': if False, speech excerpts are return labelled as 'speech'
                 if True, speech excerpts are splitted into 'male' and 'female' segments
-        """      
+        """     
+
+        if (device != 'cuda'):
+              os.environ["CUDA_DEVICE_ORDER"]= '-1'  
+        else:
+           pass       
+
+
+        import tensorflow as tf
+        
+        config = tf.compat.v1.ConfigProto()
+        config.gpu_options.allow_growth = True 
+        config.log_device_placement = True 
+        sess = tf.compat.v1.Session(config=config)
+        set_session(sess)    
+
+        
         self.complete_output = complete_output
         self.sample_rate = sr
+        self.ffmpeg_path=ffmpeg_path
+        self.input_type = input_type
+        self.device = device
+        
 
 
 #        self.graph = KB.get_session().graph # To prevent the issue of keras with tensorflow backend for async tasks
@@ -366,9 +392,10 @@ class Segmenter:
         assert detect_gender in [True, False]
         self.detect_gender = detect_gender
         if detect_gender:
-            self.gender = Gender(batch_size)
+            self.gender = Gender(batch_size , vad_type ,gender_path)
         self.vad_type = vad_type
         self.model_path = model_path
+        self.gender_path = gender_path
 
     def segment_feats(self, mspec, loge, difflen, start_sec):
         """
@@ -391,9 +418,12 @@ class Segmenter:
                 vadseg.append(('speech', start, stop))
             lseg.append((lab, start, stop))
         if (self.vad_type == 'vad'):
-           return [(lab, start_sec + start * .02, start_sec + stop * .02 , stop-start) for lab, start, stop in vadseg]
+           return [(lab, start_sec + start * .02, start_sec + stop * .02 , (stop-start) * .02) for lab, start, stop in vadseg]
         # perform voice activity detection
         lseg = self.vad(mspec, lseg, difflen)
+        
+        
+        
 
         # perform gender segmentation on speech segments
         if self.detect_gender:
@@ -401,10 +431,10 @@ class Segmenter:
         if (self.complete_output):
            return   [(lab, start_sec + start * .02, start_sec + stop * .02 , (stop-start) * .02) for lab, start, stop in lseg ]
         else:
-           return   [[lab, start_sec + start * .02, start_sec + stop * .02 , (stop-start) * .02] for lab, start, stop in lseg if (lab=='speech')]   
+           return   [[lab, start_sec + start * .02, start_sec + stop * .02 , (stop-start) * .02] for lab, start, stop in lseg if (lab=='male' or lab=="female" or lab=="speech")]   
 
 
-    def __call__(self, medianame, input_type='file',start_sec=None, stop_sec=None):
+    def __call__(self, medianame,start_sec=None, stop_sec=None):
         """
         Return segmentation of a given file
                 * convert file to wav 16k mono with ffmpeg
@@ -418,7 +448,7 @@ class Segmenter:
         """
         
         
-        mspec, loge, difflen , me = media2feats(medianame, input_type ,self.sample_rate)
+        mspec, loge, difflen , me = media2feats(medianame, self.input_type ,self.sample_rate,ffmpeg_path=self.ffmpeg_path)
         
         if start_sec is None:
             start_sec = 0
