@@ -1,2 +1,2 @@
 from .segmenter import Segmenter,filter_output,MVN,filter_sig,filter_fea
-from .export_funcs import seg2aud
\ No newline at end of file
+from .export_funcs import seg2aud,seg2Info
\ No newline at end of file
