@@ -274,17 +274,24 @@ class DnnSegmenter:
         other labels will stay unchanged
     * outlabels: the labels associated the output of neural network models
     """
-    def __init__(self, batch_size, vad_type , model_path,tq):
+    def __init__(self, batch_size, vad_type , model_path,tq, device):
         # load the DNN model
       if (vad_type!='vad'):
-        xx = SAD_INA_MODEL()       
-        xx.load_state_dict(torch.load(model_path))  
+        xx = SAD_INA_MODEL()   
+        sample_input = [torch.rand(16,68,21)]        
+        xx.load_state_dict(torch.load(model_path,map_location=torch.device(device)))  
+        xx.eval()
+        if (device!="cuda"):
+          xx = torch.jit.trace(xx, sample_input)
+          xx = torch.jit.freeze(xx)
+          
         print('model Loded from--> ' + model_path)
-        self.nn=xx.to('cuda')
+        self.nn=xx.to(device)
         self.nn.eval()    
         # self.nn.summary()
         self.batch_size = batch_size
         self.tq= tq
+        self.device= device
         
     def __call__(self, mspec, lseg):
         """
@@ -311,7 +318,7 @@ class DnnSegmenter:
        
         bsize=self.batch_size
         if len(batch) > 0:
-            batch = torch.cat(batch,0).to('cuda')
+            batch = torch.cat(batch,0).to(self.device)
             fsize = len(batch)
             num = (fsize//bsize)+1
             x=[]
@@ -377,7 +384,7 @@ class Gender(DnnSegmenter):
 class Segmenter:
 
 
-    def __init__(self, vad_type = 'sad' , vad_engine='smn', detect_gender=False, sr=8000, batch_size=32 , complete_output=False,model_path=r"C:\sad_model_pytorch.pth" , max_time=120 , tq=1):
+    def __init__(self, vad_type = 'sad' , vad_engine='smn', detect_gender=False, sr=8000, batch_size=32 , complete_output=False,model_path=r"C:\sad_model_pytorch.pth" , max_time=120 , tq=1,ffmpeg_path='c:\\ffmpeg.exe', device='cuda',input_type="file"):
         """
         Load neural network models
         
@@ -392,9 +399,12 @@ class Segmenter:
                 if True, speech excerpts are splitted into 'male' and 'female' segments
         """      
         self.sample_rate = sr
+        self.device = device
         self.complete_output = complete_output
         self.max_time = max_time #sec
         self.tq=tq
+        self.input_type = input_type
+        self.ffmpeg_path=ffmpeg_path
         
 
 
@@ -406,7 +416,7 @@ class Segmenter:
         if vad_engine == 'sm':
             self.vad = SpeechMusic(batch_size)
         elif vad_engine == 'smn':
-            self.vad = SpeechMusicNoise(batch_size , vad_type,model_path,tq)
+            self.vad = SpeechMusicNoise(batch_size , vad_type,model_path,tq , device)
 
         # load gender detection NN if required
         assert detect_gender in [True, False]
@@ -453,7 +463,7 @@ class Segmenter:
     def __call__(self, medianame, start_sec=None, stop_sec=None):
 
         
-        mspec, loge , me = media2feats(medianame,self.sample_rate)
+        mspec, loge , me = media2feats(medianame, self.input_type,self.sample_rate,ffmpeg_path=self.ffmpeg_path)
         
         if (mspec==[]):
            return [],me,[]
